require "active_record"
require "active_record/connection_adapters/postgresql_adapter"
require 'uri'

module SimplestMigrations
  class Database
    def self.migrate
      connection!
      ActiveRecord::Migration.verbose = true
      ActiveRecord::Migrator.migrate 'db/migrate', ENV['VERSION'] ? ENV['VERSION'].to_i : nil
    end

    def self.execute(cmd, db = nil)
      connection!(db)
      ActiveRecord::Base.connection.execute cmd
    end

    def self.name
      ENV['DATABASE_NAME']
    end

    def self.connection!(db = nil)
      ActiveRecord::Base.establish_connection configuration(db)
      ActiveRecord::Base
    end

    def self.configuration(db = nil)
      @config ||= {}
      db_name = db || name

      @config[db_name] ||= begin
        url = ENV["DATABASE_URL"]
        uri = URI.parse(url)

        {
          database: db_name,
          adapter: 'postgresql',
          host: uri.host,
          port: uri.port || 5432,
          encoding: 'unicode',
          pool: 5,
          username: uri.user,
          password: uri.password || ''
        }
      end

      @config[db_name]
    end

    def self.set_pg_environment
      config = configuration
      ENV['PGHOST']     = config[:host].to_s
      ENV['PGPORT']     = config[:port].to_s
      ENV['PGPASSWORD'] = config[:password].to_s
      ENV['PGUSER']     = config[:username].to_s
    end
  end
end